<?php

namespace Teknisa\DF\Modules\Saida\NFSe;

use Teknisa\DF\Modules\Saida\NFSe\Utilities\Util;
use Teknisa\Libs\Util\Environment;

class NFSePontaGrossa {
    const URL_PRODUCAO = "http://200.195.154.239/WebEloWSIss/nfse_v1_2.wsdl";
    const URL_HOMOLOGACAO = "http://200.195.154.239/WebEloWSIss/nfse_v1_2.wsdl";

    protected $connection;
    protected $environment;
    protected $newCode;

    protected $Homologa;

    protected $xmlEnviarLoteRps;
    protected $listaRps = array();
    protected $listaAuxRps = array();
    protected $itensRps = array();
    protected $xmlConsultarLoteRps;
    protected $xmlCancelarNota;

    protected $publicKey;
    protected $privateKeyId;
    protected $caminhoCertificado;
    protected $caminhoXml;
    protected $linkCertificado;
    protected $linkCertKey;
    protected $caminhoSits;
    
    protected $senhaCertificadoDigital;
    protected $SenhaAcessoWS;

    // Dados Prestador
    protected $InscricaoPrestador;
    protected $InscricaoMunicipalPrestador;
    protected $RazaoSocialPrestador;

    // Dados Lote RPS
    protected $IdLote;
    protected $NumeroLote;

    // Dados RPS
    protected $IdRps;
    protected $NumeroRps;
    protected $SerieRps;
    protected $TipoRps;
    protected $DataEmissao;
    protected $NaturezaOperacao;
    protected $RegimeEspecialTributacao;
    //protected $OptanteSimplesNacional;
    protected $IncentivadorCultural;
    protected $StatusRps;
    protected $ValorServicos;
    protected $ValorDeducoes;
    protected $ValorPis;
    protected $ValorCofins;
    protected $ValorInss;
    protected $ValorIr;
    protected $ValorCsll;
    protected $IssRetido;
    protected $Aliquota;
    protected $OutrasRetencoes;
    protected $DescontoCondicionado;
    protected $DescontoIncondicionado;
    protected $Discriminacao;
    protected $CodigoMunicipioPrestador;
    protected $CodigoMunicipioTomador;
    protected $NomeMunicipioPrestador;
    protected $NomeMunicipioTomador;
    protected $InscricaoTomador;
    protected $InscricaoMunicipalTomador;
    protected $InscricaoEstadualTomador;
    protected $RazaoSocialTomador;
    protected $EnderecoTomador;
    protected $NumeroTomador;
    protected $ComplementoTomador;
    protected $BairroTomador;
    protected $UfTomador;
    protected $CepTomador;
    protected $TelefoneTomador;
    protected $EmailTomador;

    protected $EnderecoPrestador;
    protected $NumeroPrestador;
    protected $ComplementoPrestador;
    protected $BairroPrestador;
    protected $UfPrestador;
    protected $CepPrestador;

    public function __construct($connection, Environment $environment, $newCode, $xmlDir, $certUploadDir = null, $certUploadLink = null, $certKeyLink = null) {
        $this->connection = $connection;
        $this->environment = $environment;
        $this->newCode = $newCode;
        
        $this->setCaminhoXml($xmlDir);
        if($certUploadDir) {
            $this->setCaminhoCertificado($certUploadDir);
        }
        else {
            $this->linkCertificado = $certUploadLink;
        }

        $this->caminhoSits =  realpath(__DIR__ . DIRECTORY_SEPARATOR . ".." . DIRECTORY_SEPARATOR . "SITS");
        
        //$this->SenhaAcessoWS = "2FRD9SSK";
    }

    private function setCaminhoXml($xmlDir) {
	    $xmlDir .= "org_". $this->environment->getNrOrg() . DIRECTORY_SEPARATOR;
	    $this->caminhoXml = $xmlDir;
	    if (!is_dir($this->caminhoXml)) {
            mkdir($this->caminhoXml, 0775, true);
            chmod($this->caminhoXml, 0775);
        }
	}

	private function setCaminhoCertificado($certUploadDir) {
	    if (!is_null($certUploadDir)) {
	        $certUploadDir .= "org_". $this->environment->getNrOrg() . DIRECTORY_SEPARATOR;
	        $this->caminhoCertificado = $certUploadDir;
	        if (!is_dir($this->caminhoCertificado)) {
	            mkdir($this->caminhoCertificado, 0775, true);
	            chmod($this->caminhoCertificado, 0775);
	        }
	    }
	}

    private function gerarXmlEnviarLoteRps() {
        $this->xmlEnviarLoteRps = '
        <EnviarLoteRpsEnvio xmlns="http://shad.elotech.com.br/schemas/iss/nfse_v1_2.xsd">
            <IdentificacaoRequerente>
                <CpfCnpj>
                    ';

                if (strlen($this->InscricaoPrestador) == 11) {
                    $this->xmlEnviarLoteRps .= '<Cpf>'.$this->InscricaoPrestador.'</Cpf>';
                }
                else {
                    $this->xmlEnviarLoteRps .= '<Cnpj>'.$this->InscricaoPrestador.'</Cnpj>';
                }

                $this->xmlEnviarLoteRps .= '
                </CpfCnpj>
                <InscricaoMunicipal>'.$this->InscricaoMunicipalPrestador.'</InscricaoMunicipal>
                <Senha>'.$this->SenhaAcessoWS.'</Senha>
                <Homologa>'.$this->Homologa.'</Homologa>
            </IdentificacaoRequerente>
            <LoteRps>
                <NumeroLote>'.$this->NumeroLote.'</NumeroLote>
                <QuantidadeRps>'.count($this->listaRps).'</QuantidadeRps>
                <ListaRps>';

                foreach ($this->listaRps as $rps) {
                    $this->xmlEnviarLoteRps .= "\n" . $rps;
                }

                $this->xmlEnviarLoteRps .= '
                </ListaRps>
            </LoteRps>
        </EnviarLoteRpsEnvio>';
    }
    
    /*private function gerarXmlEnviarLoteRps_Envelope() {
        $this->xmlEnviarLoteRps = '
        <SOAP-ENV:Envelope xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/">
            <SOAP-ENV:Header></SOAP-ENV:Header>
            <SOAP-ENV:Body xmlns:wsu="http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-utility-1.0.xsd" wsu:Id="id-1">
                <EnviarLoteRpsEnvio xmlns="http://shad.elotech.com.br/schemas/iss/nfse_v1_2.xsd">
                    <IdentificacaoRequerente>
                        <CpfCnpj>
                            ';

                        if (strlen($this->InscricaoPrestador) == 11) {
                            $this->xmlEnviarLoteRps .= '<Cpf>'.$this->InscricaoPrestador.'</Cpf>';
                        }
                        else {
                            $this->xmlEnviarLoteRps .= '<Cnpj>'.$this->InscricaoPrestador.'</Cnpj>';
                        }

                        $this->xmlEnviarLoteRps .= '
                        </CpfCnpj>
                        <InscricaoMunicipal>'.$this->InscricaoMunicipalPrestador.'</InscricaoMunicipal>
                        <Senha>'.$this->SenhaAcessoWS.'</Senha>
                        <Homologa>'.$this->Homologa.'</Homologa>
                    </IdentificacaoRequerente>
                    <LoteRps>
                        <NumeroLote>'.$this->NumeroLote.'</NumeroLote>
                        <QuantidadeRps>'.count($this->listaRps).'</QuantidadeRps>
                        <ListaRps>';

                        foreach ($this->listaRps as $rps) {
                            $this->xmlEnviarLoteRps .= "\n" . $rps;
                        }

                        $this->xmlEnviarLoteRps .= '
                        </ListaRps>
                    </LoteRps>
                </EnviarLoteRpsEnvio>
            </SOAP-ENV:Body>
        </SOAP-ENV:Envelope>';
    }*/

    private function adicionarRpsEnviarLote() {
        $xmlRps = '
        <DeclaracaoPrestacaoServico>
            <InfDeclaracaoPrestacaoServico>
                <Rps>
                    <IdentificacaoRps>
                        <Numero>'.$this->NumeroRps.'</Numero>
                        <Serie>'.$this->SerieRps.'</Serie>
                        <Tipo>'.$this->TipoRps.'</Tipo>
                    </IdentificacaoRps>
                    <DataEmissao>'.$this->DataEmissao.'</DataEmissao>
                    <Status>'.$this->StatusRps.'</Status>
                </Rps>
                <Competencia>'.$this->DataEmissao.'</Competencia>
                <Servico>
                    <Valores>
                        <ValorServicos>'.Util::formatarNumero($this->ValorServicos).'</ValorServicos>
                        <ValorDeducoes>'.Util::formatarNumero($this->ValorDeducoes).'</ValorDeducoes>
                        <ValorPis>'.Util::formatarNumero($this->ValorPis).'</ValorPis>
                        <ValorCofins>'.Util::formatarNumero($this->ValorCofins).'</ValorCofins>
                        <ValorInss>'.Util::formatarNumero($this->ValorInss).'</ValorInss>
                        <ValorIr>'.Util::formatarNumero($this->ValorIr).'</ValorIr>
                        <ValorCsll>'.Util::formatarNumero($this->ValorCsll).'</ValorCsll>
                        <OutrasRetencoes>'.Util::formatarNumero($this->OutrasRetencoes).'</OutrasRetencoes>
                        <Aliquota>'.$this->Aliquota.'</Aliquota>
                        <DescontoIncondicionado>'.Util::formatarNumero($this->DescontoIncondicionado).'</DescontoIncondicionado>
                        <DescontoCondicionado>'.Util::formatarNumero($this->DescontoCondicionado).'</DescontoCondicionado>
                    </Valores>
                    <IssRetido>'.$this->IssRetido.'</IssRetido>
                    <Discriminacao>'.$this->Discriminacao.'</Discriminacao>
                    <CodigoMunicipio>'.$this->CodigoMunicipioPrestador.'</CodigoMunicipio>
                    <ExigibilidadeISS>'.$this->NaturezaOperacao.'</ExigibilidadeISS>
                    <MunicipioIncidencia>'.$this->CodigoMunicipioPrestador.'</MunicipioIncidencia>
                    <ListaItensServico>';

                    foreach ($this->itensRps as $item) {
                        $xmlRps .= '
                        <ItemServico>
                            <ItemListaServico>'.$item["ItemListaServico"].'</ItemListaServico>
                            <CodigoCnae>'.$item["CodigoCnae"].'</CodigoCnae>
                            <Descricao>'.$item["Descricao"].'</Descricao>
                            <Tributavel>'.$item["Tributavel"].'</Tributavel>
                            <Quantidade>'.$item["Quantidade"].'</Quantidade>
                            <ValorUnitario>'.$item["ValorUnitario"].'</ValorUnitario>
                            <ValorDesconto>'.$item["ValorDesconto"].'</ValorDesconto>
                            <ValorLiquido>'.$item["ValorLiquido"].'</ValorLiquido>
                        </ItemServico>';
                    }

                    $xmlRps .= '
                    </ListaItensServico>
                </Servico>
                <DadosPrestador>
                    <IdentificacaoPrestador>
                        <CpfCnpj>
                            ';

                        if (strlen($this->InscricaoPrestador) == 11) {
                            $xmlRps .= '<Cpf>'.$this->InscricaoPrestador.'</Cpf>';
                        }
                        else {
                            $xmlRps .= '<Cnpj>'.$this->InscricaoPrestador.'</Cnpj>';
                        }

                    $xmlRps .= '
                        </CpfCnpj>
                        <InscricaoMunicipal>'.$this->InscricaoMunicipalPrestador.'</InscricaoMunicipal>
                    </IdentificacaoPrestador>
                    <RazaoSocial>'.$this->RazaoSocialPrestador.'</RazaoSocial>
                    <Endereco>
                        <Endereco>'.$this->EnderecoPrestador.'</Endereco>
                        <Numero>'.$this->NumeroPrestador.'</Numero>
                        <Complemento>'.$this->ComplementoPrestador.'</Complemento>
                        <Bairro>'.$this->BairroPrestador.'</Bairro>
                        <CodigoMunicipio>'.$this->CodigoMunicipioPrestador.'</CodigoMunicipio>
                        <CidadeNome>'.$this->NomeMunicipioPrestador.'</CidadeNome>
                        <Uf>'.$this->UfPrestador.'</Uf>
                        <Cep>'.$this->CepPrestador.'</Cep>
                    </Endereco>
                </DadosPrestador>
                <Tomador>
                    <IdentificacaoTomador>
                        <CpfCnpj>
                            ';

                        if (strlen($this->InscricaoTomador) == 11) {
                            $xmlRps .= '<Cpf>'.$this->InscricaoTomador.'</Cpf>';
                        }
                        else {
                            $xmlRps .= '<Cnpj>'.$this->InscricaoTomador.'</Cnpj>';
                        }

                        $xmlRps .= '
                        </CpfCnpj>
                        <InscricaoMunicipal>'.$this->InscricaoMunicipalTomador.'</InscricaoMunicipal>
                    </IdentificacaoTomador>
                    <RazaoSocial>'.$this->RazaoSocialTomador.'</RazaoSocial>
                    <Endereco>
                        <Endereco>'.$this->EnderecoTomador.'</Endereco>
                        <Numero>'.$this->NumeroTomador.'</Numero>
                        <Complemento>'.$this->ComplementoTomador.'</Complemento>
                        <Bairro>'.$this->BairroTomador.'</Bairro>
                        <CodigoMunicipio>'.$this->CodigoMunicipioTomador.'</CodigoMunicipio>
                        <CidadeNome>'.$this->NomeMunicipioTomador.'</CidadeNome>
                        <Uf>'.$this->UfTomador.'</Uf>
                        <Cep>'.$this->CepTomador.'</Cep>
                    </Endereco>
                    <Contato>
                        <Telefone>'.$this->TelefoneTomador.'</Telefone>
                        <Email>'.$this->EmailTomador.'</Email>
                    </Contato>
                    <InscricaoEstadual>'.$this->InscricaoEstadualTomador.'</InscricaoEstadual>
                </Tomador>
                <IncentivoFiscal>1</IncentivoFiscal>
            </InfDeclaracaoPrestacaoServico>
        </DeclaracaoPrestacaoServico>';

                //<RegimeEspecialTributacao>'.$this->RegimeEspecialTributacao.'</RegimeEspecialTributacao>

        /*
        <ConstrucaoCivil>
            <CodigoObra>455</CodigoObra>
            <Art>41</Art>
            <Incorporacao>2</Incorporacao>
        </ConstrucaoCivil>
        */

        $this->listaRps[] = $xmlRps;
    }

    private function getXmlAssinadoEnviarLoteRps() {
        $this->xmlEnviarLoteRps = str_replace("&", "&#38;", $this->xmlEnviarLoteRps);
        $domXml = new \DOMDocument("1.0", "UTF-8");
        $domXml->loadXML($this->xmlEnviarLoteRps, LIBXML_NOBLANKS);
        $domXml = $this->removerTagsVazias($domXml);
        $domXml->formatOutput = true;
        $xmlTemp = $domXml->saveXML();

        $domXml = new \DOMDocument("1.0", "UTF-8");
        $domXml->loadXML($xmlTemp);
        $domXml->encoding = "UTF-8";
        //$domXml = $this->assinaXml($domXml);
        if ($domXml === false) {
            throw new \Exception("Certificado digital não encontrado!");
        }
        else {
            $this->limparListaRps();
            return $domXml->saveXML();
        }
    }

    private function removerTagsVazias(\DOMDocument $xml) {
        $xpath = new \DOMXPath($xml);
        foreach($xpath->query('//*[not(node())]') as $node) {
            if (!$node->hasAttributes() && $node->nodeName != "SOAP-ENV:Header") {
                $node->parentNode->removeChild($node);
            }
        }
        return $xml;
    }

    private function limparListaRps() {
        $this->listaRps = array();
    }

    private function enviarLoteRpsWS($xml, $cdempresa, $inscricao, $numeroLote) {
        $nomeFuncao = "EnviarLoteRps";
        $prefixoArquivo = $cdempresa.$inscricao.$numeroLote."_".$nomeFuncao."_";
        $nomeArquivoEnvio = $prefixoArquivo."env.xml";
        file_put_contents($this->caminhoXml . DIRECTORY_SEPARATOR . $nomeArquivoEnvio, $xml);
        
        try {
            $resultado = $this->enviarParaWebservice($nomeArquivoEnvio, $xml);
        }
        catch (\Exception $ex) {
            throw new \Exception($ex->getMessage());
        }
        
        $nomeArquivoResposta = $prefixoArquivo."resp.xml";
        file_put_contents($this->caminhoXml . DIRECTORY_SEPARATOR . $nomeArquivoResposta, $resultado);

        if ($resultado) {
            $domResult = new \DOMDocument("1.0", "UTF-8");
            $domResult->loadXML($resultado);
            
            if ($domResult->getElementsByTagName("NumeroLote")->length > 0) {

                $protocolo = $domResult->getElementsByTagName("NumeroLote")->item(0)->nodeValue;

                foreach ($this->listaAuxRps as $nota) {
                    Util::atualizarNotaTransmissao($this->connection, $nota["CDSERIERPS"], $nota["NRRPS"], $nota["CDEMPLOTERPS"], $nota["CDINSCESTA"], $nota["CDINSCMUNI"], $nota["NRSEQLOTERPS"], $nota["CDFILIAL"], $nota["NRLANCTONF"], $this->environment->getNrOrg(), $this->environment->getCdOperador());
                }
                
                Util::atualizarLote($this->connection, $this->listaAuxRps[0]["CDEMPLOTERPS"], $this->listaAuxRps[0]["CDINSCESTA"], $this->listaAuxRps[0]["CDINSCMUNI"], $this->NumeroLote, 2, $protocolo, $this->environment->getNrOrg(), $this->environment->getCdOperador());
                
                return true;
            }
            else if ($domResult->getElementsByTagName("MensagemRetorno")->length > 0) {
                $erros = array("TIPO" => 2);
                foreach ($domResult->getElementsByTagName("MensagemRetorno") as $erro) {
                    $nodeIdentificacaoRps = $erro->getElementsByTagName("IdentificacaoRps")->item(0);
                    $erros[] = array(
                        "CODIGO" => Util::getChildNodeByName($erro, "Codigo")->nodeValue,
                        "DESCRICAO" => Util::getChildNodeByName($erro, "Mensagem")->nodeValue . " " . Util::getChildNodeByName($erro, "Correcao")->nodeValue,
                        "NRRPS"  => Util::getChildNodeByName($nodeIdentificacaoRps, "Numero")->nodeValue
                    );
                    
                }
                return $erros;
            }
        }
        else {
            return false;
        }
    }

    private function gerarXmlConsultarLoteRps($inscricaoJuridica, $inscricaoMunicipal, $protocolo) {
        $this->xmlConsultarLoteRps = '<ConsultarLoteRpsEnvio xmlns="http://shad.elotech.com.br/schemas/iss/nfse_v1_2.xsd">
            <IdentificacaoRequerente>
                <CpfCnpj>
                    ';
                    
                    if (strlen($inscricaoJuridica) == 11) {
                        $this->xmlConsultarLoteRps .= '<Cpf>'.$inscricaoJuridica.'</Cpf>';
                    }
                    else {
                        $this->xmlConsultarLoteRps .= '<Cnpj>'.$inscricaoJuridica.'</Cnpj>';
                    }
                    
                $this->xmlConsultarLoteRps .= '
                </CpfCnpj>
                <InscricaoMunicipal>'.$inscricaoMunicipal.'</InscricaoMunicipal>
                <Senha>'.$this->SenhaAcessoWS.'</Senha>
                <Homologa>'.$this->Homologa.'</Homologa>
            </IdentificacaoRequerente>
            <NumeroLote>'.$protocolo.'</NumeroLote>
        </ConsultarLoteRpsEnvio>';
        
        return $this->xmlConsultarLoteRps;
    }

    public function consultarLoteRps($lote) {
        $this->cdempresa = $lote["CDEMPLOTERPS"];
        $this->cdinscmuni = str_replace(array(".","-","/"," "), "", $lote["CDINSCMUNI"]);
        
        $ambiente = Util::getAmbiente($this->connection, $lote["CDEMPLOTERPS"], $lote["CDINSCMUNI"], $this->environment->getNrOrg());
        $this->Homologa = ($ambiente == 1) ? false : true;

        $this->SenhaAcessoWS = Util::getSenhaAcessoWS($connection, $lote["CDEMPLOTERPS"], $lote["CDINSCMUNI"], $this->environment->getNrOrg());

        //$this->RazaoSocialPrestador = $lote["NMRAZSOCEMPR"];
        $inscricaoJuridica = str_replace(array("-", "/", ".", " "), "", $lote["NRINSJURIEST"]);
        $inscricaoMunicipal = str_replace(array("-", "/", ".", " "), "", $lote["CDINSCMUNI"]);
        $xml = $this->gerarXmlConsultarLoteRps($inscricaoJuridica, $inscricaoMunicipal, $lote["NRPROTOCOLORPS"]);
        $resultado = $this->consultarLoteRpsWS($xml, $lote["CDEMPLOTERPS"], $lote["NRINSJURIEST"], $lote["NRSEQLOTERPS"]);

        if (is_string($resultado)) {
            $domResult = new \DOMDocument("1.0", "UTF-8");
            $domResult->loadXML($resultado);

            if ($domResult->getElementsByTagName("ListaNfse")->length > 0) {

                $nodeList = $domResult->getElementsByTagName("InfNfse");
                
                if ($nodeList->length > 0) {
                    foreach ($nodeList as $nota) {
                        $numeroNota = Util::getChildNodeByName($nota, "Numero")->nodeValue;
                        $codigoVerificacao = Util::getChildNodeByName($nota, "CodigoVerificacao")->nodeValue;
                        $chaveAcesso = Util::getChildNodeByName($nota, "ChaveAcesso")->nodeValue;
                        
                        /*==================================*/
                        $dataEmissao = dfs::getChildNodeByName($nota, "DataEmissao")->nodeValue;
                        $dataEmissao = date_create_from_format("Y-m-d", $dataEmissao);
                        $dataEmissao = $dataEmissao->format("d/m/Y");
                        
                        $competencia = $nota->getElementsByTagName("Competencia")->item(0)->nodeValue;
                        $competencia = date_create_from_format("Y-m-d", $competencia);
                        $competencia = $competencia->format("d/m/Y");
                        /*==================================*/
                        
                        $nodeIdentificacaoRps = $nota->getElementsByTagName("IdentificacaoRps")->item(0);
                        $serieRps = Util::getChildNodeByName($nodeIdentificacaoRps, "Serie")->nodeValue;
                        $numeroRps = Util::getChildNodeByName($nodeIdentificacaoRps, "Numero")->nodeValue;
                        $numeroRps = str_pad($numeroRps, 15, "0", STR_PAD_LEFT);
    
                        Util::atualizarNumeroRps($this->connection, $serieRps, $numeroRps, $lote["CDEMPLOTERPS"], $lote["CDINSCESTA"], $lote["CDINSCMUNI"], $lote["NRSEQLOTERPS"], $numeroNota, $codigoVerificacao, 4, $dataEmissao, $competencia, $this->environment->getNrOrg(), $this->environment->getCdOperador());
    
                        Util::atualizarNotaConsulta($this->connection, $serieRps, $numeroRps, $lote["CDEMPLOTERPS"], $lote["CDINSCESTA"], $lote["CDINSCMUNI"], $lote["NRSEQLOTERPS"], $numeroNota, $this->environment->getNrOrg(), $this->environment->getCdOperador());
                        
                        Util::inserirArquivoXml($this->connection, $serieRps, $numeroRps, $lote["CDEMPLOTERPS"], $lote["CDINSCESTA"], $lote["CDINSCMUNI"], $lote["NRSEQLOTERPS"], "A", $resultado, $this->environment->getNrOrg(), $this->environment->getCdOperador());
                    }
                    
                    Util::atualizarLote($this->connection, $lote["CDEMPLOTERPS"], $lote["CDINSCESTA"], $lote["CDINSCMUNI"], $lote["NRSEQLOTERPS"], 4, $lote["NRPROTOCOLORPS"], $this->environment->getNrOrg(), $this->environment->getCdOperador());
                    
                    return true;
                }
                else {
                    return null;
                }
            }
            else {
                $erros = array();
                $nodeList = $domResult->getElementsByTagName("MensagemRetornoLote");
                foreach ($nodeList as $erro) {
                    $codigoErro = Util::getChildNodeByName($erro, "Codigo")->nodeValue;
                    $descricaoErro = Util::getChildNodeByName($erro, "Mensagem")->nodeValue;
                    
                    $erros[] = array(
                        "CODIGO" => $codigoErro,
                        "DESCRICAO" => $descricaoErro
                    );
                    
                    // E92 = lote ainda nao foi processado
                    if ($codigoErro == "E178") {
                        return $erros;
                    }
                }
                
                Util::atualizarSituacaoNumeroRpsPeloLote($this->connection, $lote["CDSERIERPS"], $lote["CDEMPLOTERPS"], $lote["CDINSCESTA"], $lote["CDINSCMUNI"], $lote["NRSEQLOTERPS"], 3, $this->environment->getNrOrg(), $this->environment->getCdOperador());
                
                Util::atualizarLote($this->connection, $lote["CDEMPLOTERPS"], $lote["CDINSCESTA"], $lote["CDINSCMUNI"], $lote["NRSEQLOTERPS"], 3, $lote["NRPROTOCOLORPS"], $this->environment->getNrOrg(), $this->environment->getCdOperador());
                
                return $erros;
            }
        }
        else {
            return false;
        }
    }

    private function consultarLoteRpsWS($xml, $cdempresa, $inscricao, $numeroLote) {
        $nomeFuncao = "ConsultarLoteRps";
        $prefixoArquivo = $cdempresa.$inscricao.$numeroLote."_".$nomeFuncao."_";
        $nomeArquivoEnvio = $prefixoArquivo."env.xml";
        file_put_contents($this->caminhoXml . DIRECTORY_SEPARATOR . $nomeArquivoEnvio, $xml);
        
        try {
            $resultado = $this->enviarParaWebservice($nomeArquivoEnvio, $xml);
        }
        catch (\Exception $ex) {
            throw new \Exception($ex->getMessage());
        }
        
        $nomeArquivoResposta = $prefixoArquivo."resp.xml";
        file_put_contents($this->caminhoXml . DIRECTORY_SEPARATOR . $nomeArquivoResposta, $resultado);
        return $resultado;
    }

    public function cancelarNotas($notas, $codMotivo, $dsMotivo, $cdmunicibge, $cdmunicsiafi) {
        
        $ambiente = Util::getAmbiente($this->connection, $notas[0]["CDEMPRESA"], $notas[0]["CDINSCMUNI"], $this->environment->getNrOrg());
        $this->Homologa = ($ambiente == 1) ? false : true;

        $this->senhaCertificadoDigital = Util::getSenhaCertificadoDigital($this->connection, $notas[0]["CDEMPRESA"], $notas[0]["CDINSCMUNI"], $this->environment->getNrOrg());

        $this->SenhaAcessoWS = Util::getSenhaAcessoWS($connection, $notas[0]["CDEMPRESA"], $notas[0]["CDINSCMUNI"], $this->environment->getNrOrg());

        //$nota = $nota[0];
        $errosTotais = array();
        foreach ($notas as $nota) {
            $this->gerarXmlCancelarNota($nota["NRNOTAFISC"], $nota["NRACESSONFE"], $nota["NRINSJURIEST"], $nota["CDINSCMUNI"], $codMotivo);
            $xml = $this->getXmlAssinadoCancelarNota();
    
            $resultado = $this->cancelarNotaWS($xml, $nota["CDEMPRESA"], $nota["NRINSJURIEST"], $nota["NRNOTAFISC"]);
    
            if (is_string($resultado)) {
                $domResult = new \DOMDocument("1.0", "UTF-8");
                $domResult->loadXML($resultado);
                $nodeList = $domResult->getElementsByTagName("MensagemRetorno");
                if ($nodeList->length > 0) {
                    $errosNota = array();
                    foreach ($nodeList as $erro) {
                        $errosNota[] = array(
                            "CODIGO" => Util::getChildNodeByName($erro, "Codigo")->nodeValue,
                            "DESCRICAO" => Util::getChildNodeByName($erro, "Mensagem")->nodeValue . " " . Util::getChildNodeByName($erro, "Correcao")->nodeValue
                        );
                    }
                    $errosTotais[$nota["NRLANCTONF"]] = $errosNota;
                }
                else {
                   $motivos = array(
                        1 => "Erro na emissão",
                        2 => "Serviço não prestado",
                        3 => "Erro na assinatura", 
                        4 => "Duplicidade da nota",
                        5 => "Erro no Processamento"
                    );
                    $descMotivo = strtr($codMotivo, $motivos);
    
                    Util::inserirNotaCancelada($this->connection, "S", "C", $descMotivo, "C", $nota["CDFILIAL"], $nota["NRLANCTONF"], $this->environment->getNrOrg(), $this->newCode);
                    
                    Util::atualizarNotaCancelada($this->connection, "N", $nota["CDFILIAL"], $nota["NRLANCTONF"], $this->environment->getNrOrg());
                }
            }
            else {
                return false;
            }
        }
        return (count($errosTotais) > 0) ? $errosTotais : true;
    }

    private function gerarXmlCancelarNota($nrnotafisc, $chaveAcesso, $inscricaoJuridica, $inscricaoMunicipal, $codMotivoCancelamento) {
        
        $this->xmlCancelarNota = '<CancelarNfseEnvio xmlns="http://shad.elotech.com.br/schemas/iss/nfse_v1_2.xsd">
            <IdentificacaoRequerente>
                <CpfCnpj>
                    ';
        
                if (strlen($inscricaoJuridica) == 11) {
                    $this->xmlCancelarNota .= '<Cpf>'.$inscricaoJuridica.'</Cpf>';
                }
                else {
                    $this->xmlCancelarNota .= '<Cnpj>'.$inscricaoJuridica.'</Cnpj>';
                }
        
        $this->xmlCancelarNota .= '</CpfCnpj>
                <InscricaoMunicipal>'.$inscricaoMunicipal.'</InscricaoMunicipal>
                <Senha>'.$this->SenhaAcessoWS.'</Senha>
                <Homologa>'.$this->Homologa.'</Homologa>
            </IdentificacaoRequerente>
            <Pedido>
                <InfPedidoCancelamento>
                    <Numero>'.$nrnotafisc.'</Numero>
                    <ChaveAcesso>'.$chaveAcesso.'</ChaveAcesso>
                    <CodigoCancelamento>'.$codMotivoCancelamento.'</CodigoCancelamento>
                </InfPedidoCancelamento>
            </Pedido>
        </CancelarNfseEnvio>';
    }

    private function getXmlAssinadoCancelarNota() {
        $this->xmlCancelarNota = str_replace("&", "&#38;", $this->xmlCancelarNota);

        $domXml = new \DOMDocument("1.0", "UTF-8");
        $domXml->loadXML($this->xmlCancelarNota, LIBXML_NOBLANKS);
        $domXml = $this->removerTagsVazias($domXml);
        $domXml->formatOutput = true;
        $xmlTemp = $domXml->saveXML();

        $domXml = new \DOMDocument("1.0", "UTF-8");
        $domXml->loadXML($xmlTemp);
        $domXml->encoding = "UTF-8";
        //$domXml = $this->assinaXml($domXml);
        if ($domXml === false) {
            throw new \Exception("Certificado digital não encontrado!");
        }
        else {
            return $domXml->saveXML();
        }
    }

    private function cancelarNotaWS($xml, $cdempresa, $inscricao, $nrnotafisc) {
        $nomeFuncao = "CancelarNfse";
        $prefixoArquivo = $cdempresa.$inscricao.$nrnotafisc."_".$nomeFuncao."_";
        $nomeArquivoEnvio = $prefixoArquivo."env.xml";
        file_put_contents($this->caminhoXml . DIRECTORY_SEPARATOR . $nomeArquivoEnvio, $xml);
        
        try {
            $resultado = $this->enviarParaWebservice($nomeArquivoEnvio, $xml);
        }
        catch (\Exception $ex) {
            throw new \Exception($ex->getMessage());
        }
        
        $nomeArquivoResposta = $prefixoArquivo."resp.xml";
        file_put_contents($this->caminhoXml . DIRECTORY_SEPARATOR . $nomeArquivoResposta, $resultado);
        
        return $resultado;
    }

    private function enviarParaWebservice($nomeArquivoEnvio, $conteudo) {
        //$caminho = $this->caminhoCertificado . $this->cdempresa . "_" . $this->cdinscmuni . DIRECTORY_SEPARATOR . "certKey.pem";
        
        $files = glob($this->caminhoSits . DIRECTORY_SEPARATOR . "Enviar" . DIRECTORY_SEPARATOR . "*"); // get all file names
        foreach($files as $file) { // iterate files
          if (is_file($file))
            unlink($file); // delete file
        }
        
        file_put_contents($this->caminhoSits . DIRECTORY_SEPARATOR . "Enviar" . DIRECTORY_SEPARATOR . $nomeArquivoEnvio, $conteudo);
        chmod($this->caminhoSits . DIRECTORY_SEPARATOR . "Enviar" . DIRECTORY_SEPARATOR . $nomeArquivoEnvio, 0777);
        
        $validacaoXml = $this->validarXml($this->caminhoSits . DIRECTORY_SEPARATOR . "Enviar" . DIRECTORY_SEPARATOR . $nomeArquivoEnvio, "http://shad.elotech.com.br/schemas/iss/nfse_v1_2.xsd");
        if (!empty($validacaoXml)) {
            throw new \Exception("Falha na validação do XML:<br/>" . $validacaoXml);
        }
        
        $comando = shell_exec("java -jar " . $this->caminhoSits . DIRECTORY_SEPARATOR . "WSProxy.jar");
        
        if (!is_file($this->caminhoSits . DIRECTORY_SEPARATOR . "Recebido" . DIRECTORY_SEPARATOR . "resp_".$nomeArquivoEnvio)) {
            throw new \Exception("Falha na comunicação com a prefeitura.");
        }
        
        $xmlRetorno = file_get_contents($this->caminhoSits . DIRECTORY_SEPARATOR . "Recebido" . DIRECTORY_SEPARATOR . "resp_".$nomeArquivoEnvio);
        
        return $xmlRetorno;
    }
    
    private function validarXml($caminhoXml, $caminhoXsd) {
        libxml_use_internal_errors(true);
        $erro = null;
        
        $xml = new \DOMDocument();
        $xml->load($caminhoXml);
        
        if (!$xml->schemaValidate($caminhoXsd)) {
            //foreach (libxml_get_errors() as $error) {}
            $erro = libxml_get_errors()[0]->message;
        }
        
        libxml_clear_errors();
        libxml_use_internal_errors(false);
        
        return $erro;
    }
    
    public function enviarLoteRps($notas) {
        $erros = array("TIPO" => 1);
        $this->listaAuxRps = array();
        
        $nrlanctonf = $notas[0]["NRLANCTONF"];
        $cdfilial = $notas[0]["CDFILIAL"];
        $notafiscal = Util::getCorpoNF($this->connection, $nrlanctonf, $cdfilial, $this->environment->getNrOrg());

        $this->cdempresa = $notafiscal["CDEMPRESA"];
        $this->cdinscmuni = str_replace(array(".","-","/"," "), "", $notafiscal["CDINSCMUNIEMP"]);

        $ambiente = Util::getAmbiente($this->connection, $notafiscal["CDEMPRESA"], $notafiscal["CDINSCMUNIEMP"], $this->environment->getNrOrg());
        $this->Homologa = ($ambiente == 1) ? false : true;

        $this->senhaCertificadoDigital = Util::getSenhaCertificadoDigital($this->connection, $notafiscal["CDEMPRESA"], $notafiscal["CDINSCMUNIEMP"], $this->environment->getNrOrg());

        $this->SenhaAcessoWS = Util::getSenhaAcessoWS($connection, $notafiscal["CDEMPRESA"], $notafiscal["CDINSCMUNIEMP"], $this->environment->getNrOrg());

        $this->NumeroLote = Util::inserirLoteRps($this->connection, $notafiscal["CDEMPRESA"], $notafiscal["CDINSCESTA"], $notafiscal["CDINSCMUNIEMP"], $notafiscal["NRINSJURIEST"], $this->environment->getNrOrg(), $this->environment->getCdOperador(), $this->newCode);
        //$this->IdLote = "LOTE_".$notafiscal["CDEMPRESA"]."_".$this->NumeroLote;
        $this->InscricaoPrestador = $notafiscal["NRINSJURIEST"];
        $this->InscricaoPrestador = str_replace(array(".","-","/"), "", $this->InscricaoPrestador);
        $this->InscricaoMunicipalPrestador = str_replace(array(".","-","/"), "", $notafiscal["CDINSCMUNIEMP"]);
        if ($this->InscricaoMunicipalPrestador == "ISENTO") {
            $this->InscricaoMunicipalPrestador = "";
        }
        
        $this->RazaoSocialPrestador = $notafiscal["NMRAZSOCEMIT"];

        $this->EnderecoPrestador = substr($notafiscal["DSENDEEMP"], 0, 125);
        $this->NumeroPrestador = ".";
        $this->ComplementoPrestador = substr($notafiscal["DSCOMPENDEMP"], 0, 60);
        $this->BairroPrestador = substr($notafiscal["NMBAIREMP"], 0, 60);
        $this->CodigoMunicipioPrestador = $notafiscal["CDMUNICIBGEEMP"];
        $this->NomeMunicipioPrestador = substr($notafiscal["NMMUNICIPIOEMP"], 0, 125);
        $this->UfPrestador = $notafiscal["SGESTADOEMP"];
        $this->CepPrestador = substr(str_replace(array(".","-"), "", $notafiscal["NRCEPEMP"]), 0, 8);

        foreach ($notas as $nota) {
            $nrlanctonf = $nota["NRLANCTONF"];
            $cdfilial = $nota["CDFILIAL"];
            $notafiscal = Util::getCorpoNF($this->connection, $nrlanctonf, $cdfilial, $this->environment->getNrOrg());

            //VALIDACAO
            if (empty($notafiscal["CDINSCMUNIEMP"])) {
                $erros[] = array(
                    "CODIGO" => $nota["NRLANCTONF"],
                    "DESCRICAO" => "Inscrição municipal deve ser relacionada com a empresa informada."
                );
            }
            //VALIDACAO
            if ($notafiscal["IDNATUOPEREMPR"] == "99") {
                $erros[] = array(
                    "CODIGO" => $nota["NRLANCTONF"],
                    "DESCRICAO" => "Natureza da operação deve ser informada."
                );
            }
            //VALIDACAO
            if ($notafiscal["IDREGESPTRIBEMP"] == "99") {
                $erros[] = array(
                    "CODIGO" => $nota["NRLANCTONF"],
                    "DESCRICAO" => "Regime especial de tributação deve ser informado."
                );
            }

            $this->NaturezaOperacao = (int)$notafiscal["IDNATUOPEREMPR"];
            //$this->OptanteSimplesNacional = 2;

            if ($notafiscal["IDOPTSIMPLES"] == "S") {
                $this->RegimeEspecialTributacao = 7;
                //$this->OptanteSimplesNacional = 1;
            }
            else {
                $this->RegimeEspecialTributacao = (int)$notafiscal["IDREGESPTRIBEMP"];
            }
            
            //$this->IncentivadorCultural = 2;
            
            $dataAtual = date_create();
            //$this->DataEmissao = $dataAtual->format("Y-m-d\TH:i:s");
            $this->DataEmissao = $notafiscal["DTENTRSAID"] . "T" . $dataAtual->format("H:i:s");

            $this->NumeroRps = Util::inserirNumeroRps($this->connection, $this->NumeroLote, 2, $notafiscal["CDEMPRESA"], $notafiscal["CDSERIERPS"], null, $notafiscal["CDINSCESTA"], $notafiscal["CDINSCMUNIEMP"], $notafiscal["NRINSJURIEST"], null, null, $this->environment->getNrOrg(), $this->environment->getCdOperador(), $this->newCode);
            $this->SerieRps = $notafiscal["CDSERIERPS"];
            //$this->IdRps = "RPS_" . $this->SerieRps . "_" . $this->NumeroRps;
            $this->TipoRps = 1;
            $this->StatusRps = 1;

            $this->InscricaoTomador = $notafiscal["NRINSJURCLIE"];
            $this->InscricaoTomador = str_replace(array(".","-","/"," "), "", $this->InscricaoTomador);
            
            $this->InscricaoMunicipalTomador = str_replace(array(".","-","/"," "), "", $notafiscal["NRINSMUNICLIE"]);
            if (strtoupper($this->InscricaoMunicipalTomador) == "ISENTO" || strtoupper($this->InscricaoMunicipalTomador) == "ISENTA") {
                $this->InscricaoMunicipalTomador = "";
            }
            $this->InscricaoEstadualTomador = str_replace(array(".","-","/"), "", $notafiscal["NRINSESTCLIE"]);
            $this->RazaoSocialTomador = $notafiscal["NMRAZSOCCLIE"];
            $this->EnderecoTomador = substr($notafiscal["DSENDECLIE"], 0, 125);
            $this->NumeroTomador = ".";
            $this->ComplementoTomador = substr($notafiscal["DSCOMPENDCLI"], 0, 60);
            $this->BairroTomador = substr($notafiscal["NMBAIRCLIE"], 0, 60);
            $this->CodigoMunicipioTomador = $notafiscal["CDMUNICIBGECLIE"];
            $this->NomeMunicipioTomador = substr($notafiscal["NMMUNICIPIOCLIE"], 0, 125);
            $this->UfTomador = $notafiscal["SGESTADOCLIE"];
            $this->CepTomador = substr(str_replace(array(".", "-"), "", $notafiscal["NRCEPCLIE"]), 0, 8);
            $this->TelefoneTomador = substr(str_replace(array("(", ")", "-", " "), "", $notafiscal["NRTELECLIE"]), 0, 20);
            $this->EmailTomador = substr($notafiscal["DSEMAILCLIENFE"], 0, 80);

            $this->ValorServicos = $notafiscal["VRPRODUTOS"];
            //$valorLiquido = $notafiscal["VRPRODUTOS"];
            $this->ValorDeducoes = 0.0;
            $this->ValorPis = 0.0;
            $this->ValorCofins = 0.0;
            $this->ValorInss = 0.0;
            $this->ValorIr = 0.0;
            $this->ValorCsll = 0.0;
            $this->IssRetido = 2;
            //$this->ValorIssRetido = 0.0;
            $this->OutrasRetencoes = 0.0;
            $this->DescontoCondicionado = 0.0;
            $this->DescontoIncondicionado = 0.0;
            //$this->BaseCalculo = 0.0;

            $impostos = Util::getImpostos($this->connection, $nrlanctonf, $cdfilial, $this->environment->getNrOrg());

            foreach ($impostos as $imp) {
                if (in_array($imp["CDIMPOSTONFE"], array("50", "55", "56")) && $imp["IDTRATIMPO"] == "R") {
                    $this->ValorPis = (float)$imp["VALORIMPOSTO"];
                    //$valorLiquido -= $this->ValorPis;
                }
                else if (in_array($imp["CDIMPOSTONFE"], array("60", "65", "66")) && $imp["IDTRATIMPO"] == "R") {
                    $this->ValorCofins = (float)$imp["VALORIMPOSTO"];
                    //$valorLiquido -= $this->ValorCofins;
                }
                else if ($imp["CDIMPOSTONFE"] == "42" && $imp["IDTRATIMPO"] == "R") {
                    $this->ValorInss = (float)$imp["VALORIMPOSTO"];
                    //$valorLiquido -= $this->ValorInss;
                }
                else if (in_array($imp["CDIMPOSTONFE"], array("22", "25")) && $imp["IDTRATIMPO"] == "R") {
                    $this->ValorIr = (float)$imp["VALORIMPOSTO"];
                    //$valorLiquido -= $this->ValorIr;
                }
                else if (in_array($imp["CDIMPOSTONFE"], array("70", "75", "76")) && $imp["IDTRATIMPO"] == "R") {
                    $this->ValorCsll = (float)$imp["VALORIMPOSTO"];
                    //$valorLiquido -= $this->ValorCsll;
                }
                else if (in_array($imp["CDIMPOSTONFE"], array("31", "35", "36"))) {
                    if ($imp["IDTRATIMPO"] == "R") {
                        $this->IssRetido = 1;
                        //$this->ValorIssRetido = (float)$imp["VALORIMPOSTO"];
                        //$valorLiquido -= $this->ValorIssRetido;
                    }
                    //else {
                    //    $this->BaseCalculo = $imp["BASECALCULO"];
                    //}
                }
            }

            $outrasRetencoes = Util::getOutrasRetencoes($this->connection, $nrlanctonf, $cdfilial, $this->environment->getNrOrg());
            if ($outrasRetencoes["VRIMPOSTOIT"] > 0) {
                $this->OutrasRetencoes = $outrasRetencoes["VALORIMPOSTO"];
                //$valorLiquido -= $this->OutrasRetencoes;
            }

            //$this->ValorLiquidoNfse = $valorLiquido;
            
            if (count($impostos) > 0) {
                $this->Aliquota = "";
                $impostoItem = Util::getImpostoItemMaiorAliquota($this->connection, $nrlanctonf, $cdfilial, "31", $this->environment->getNrOrg());
                if (!empty($impostoItem)) {
                    $this->Aliquota = Util::formatarNumero((float)$impostoItem["VRALIQITEM"]);
                    if ((int)$this->Aliquota == 0) $this->Aliquota = "";
                }
            }
            else {
                //VALIDACAO
                $erros[] = array(
                    "CODIGO" => $nota["NRLANCTONF"],
                    "DESCRICAO" => "Impostos da nota devem ser informados."
                );
            }

            $itemProduto = Util::getItemProduto($this->connection, $nrlanctonf, $cdfilial, $this->environment->getNrOrg());
            $listaCdarvprod = array();

            $descricaoProd = "";

            foreach ($itemProduto as $linha) {
                $descricaoProd .= $linha["DSPRODEMIS"] . "\r\n";
                $listaCdarvprod[] = $linha["CDARVPROD"];
            }

            $itemEncargo = Util::getItemEncargo($this->connection, $nrlanctonf, $cdfilial, $this->environment->getNrOrg());
            //$descricaoEncargo = "";
            foreach ($itemEncargo as $item) {
                //$descricaoEncargo .= $item["DESCRICAO"] . "\n";
                if ($item["IDENCARGO"] == "D") {
                    $this->DescontoIncondicionado += (float)$item["VRITEMENCA"];
                }
            }

            /*$descricaoDadosAdic = "";
            $dadosAdicionais = Util::getDadosAdicionais($this->connection, $nrlanctonf, $cdfilial, $this->environment->getNrOrg());
            foreach ($dadosAdicionais as $item) {
                $descricaoDadosAdic .= $item["DSDADOADIC"];
            }

            $this->Discriminacao = $descricaoProd;
            if (!empty($descricaoEncargo)) {
                $this->Discriminacao .= "|" . $descricaoEncargo;
            }
            if (!empty($descricaoDadosAdic)) {
                $this->Discriminacao .= "|" . $descricaoDadosAdic;
            }*/

            $this->Discriminacao = trim($descricaoProd);
            $this->Discriminacao = substr($this->Discriminacao, 0, 2000);
            
            $this->itensRps = array();
            $itemListaServico = null;
			$codigoCnae = null;
			//$codigoCnae = $notafiscal["CDCNAE"];
			
			foreach ($itemProduto as $indice => $item) {
			    
                $codTribMuni = Util::getCodTribMuni($this->connection, $notafiscal["CDEMPRESA"], $notafiscal["CDINSCMUNIEMP"], $listaCdarvprod[$indice], $this->environment->getNrOrg());
                //VALIDACAO
                if (!is_array($codTribMuni) || empty($codTribMuni["CDCLASSERVPADR"])) {
                    $erros[] = array(
                        "CODIGO" => $nota["NRLANCTONF"],
                        "DESCRICAO" => "Código da classificação do serviço deve ser informado."
                    );
                    break;
                }
                else {
                    $codigoCnae = $codTribMuni["CDCLASSERVPADR"];
                    $itemListaServico = $codTribMuni["CDTRIBPRODPADR"];
                }
			    
                $impostoItem = Util::getImpostosItem($this->connection, $nrlanctonf, $cdfilial, $this->environment->getNrOrg(), "31", $item["NRSEQITPROD"]);
                
                if (count($impostoItem) > 0) {
                    $discriminacaoServico = substr(trim($item["DSPRODEMIS"]), 0, 20);
    				$quantidade = (float)$item["QTITEMPROD"];
                    $valorUnitario = Util::formatarNumero($item["VRUNITITEM"]);
                    $ValorDesconto = Util::formatarNumero($item["VRDESCITEM"]);
                    $valorLiquido = Util::formatarNumero($item["VRLIQUIDO"]);
                    
                    //$tributavel = ($impostoItem[0]["CDFISCALIT"] == "1") ? "S" : "N";
                    $tributavel = (int)$impostoItem[0]["CDFISCALIT"];
                        
    				$this->itensRps[] = array(
    					"ItemListaServico" => $itemListaServico,
                        "CodigoCnae"       => $codigoCnae,
                        "Descricao"        => $discriminacaoServico,
                        "Tributavel"       => $tributavel,
                        "Quantidade"       => $quantidade,
                        "ValorUnitario"    => $valorUnitario,
                        "ValorDesconto"    => $ValorDesconto,
                        "ValorLiquido"     => $valorLiquido
    				);
                }
                else {
                    //VALIDACAO
                    break;
                }
			}

            $this->adicionarRpsEnviarLote();
            
            $this->listaAuxRps[] = array(
                "CDSERIERPS"   => $notafiscal["CDSERIERPS"],
                "NRRPS"        => $this->NumeroRps,
                "CDEMPLOTERPS" => $notafiscal["CDEMPRESA"],
                "CDINSCESTA"   => $notafiscal["CDINSCESTA"],
                "CDINSCMUNI"   => $notafiscal["CDINSCMUNIEMP"],
                "NRSEQLOTERPS" => $this->NumeroLote,
                "NRINSJURIEST" => $notafiscal["NRINSJURIEST"],
                "CDFILIAL"     => $cdfilial,
                "NRLANCTONF"   => $nrlanctonf
            );
        }

        if (count($erros) > 1) {
            $this->limparListaRps();
            return $erros;
        }
        else {
            $this->gerarXmlEnviarLoteRps();
            $xml = $this->getXmlAssinadoEnviarLoteRps();
            return $this->enviarLoteRpsWS($xml, $notafiscal["CDEMPRESA"], $notafiscal["NRINSJURIEST"], $this->NumeroLote);
        }
    }
    

    private function assinaXml(\DOMDocument $domXml, array $tags) {
	    $local = ($this->caminhoCertificado);
        if($local) {
            $dir = $this->caminhoCertificado . $this->cdempresa . "_" . $this->cdinscmuni;
            if(!is_dir($dir)) return false;
            if((!is_file($dir . DIRECTORY_SEPARATOR . "certificado.pfx")) && (!is_file($dir . DIRECTORY_SEPARATOR . "certificado.p12"))) return false;
            else {
                if(is_file($dir . DIRECTORY_SEPARATOR . "certificado.pfx")) $data = file_get_contents($dir . DIRECTORY_SEPARATOR . "certificado.pfx");
                else if(is_file($dir . DIRECTORY_SEPARATOR . "certificado.p12")) $data = file_get_contents($dir . DIRECTORY_SEPARATOR . "certificado.p12");
            }
        }
        else {
            try{
                $data = file_get_contents($this->linkCertificado);
            }
            catch(\Exception $e) {
                return false;
            }
            if(empty($data)) return false;
        }
        
        openssl_pkcs12_read($data, $cert, $this->senhaCertificadoDigital);

        if (is_null($cert)) {
            return false;
        }
        else {
            file_put_contents($dir.DIRECTORY_SEPARATOR."certKey.pem", $cert["pkey"]."\r\n".$cert["cert"]);
            //file_put_contents($dir.DIRECTORY_SEPARATOR."priKey.pem", $cert["pkey"]);
            //file_put_contents($dir.DIRECTORY_SEPARATOR."pubKey.pem", $cert["cert"]);

            $this->publicKey = $this->getPublicKey($cert["cert"]);
            $this->privateKeyId = openssl_get_privatekey($cert["pkey"]);

            foreach ($tags as $tag) {
                $domXml = $this->assinaXmlAux($domXml, $tag);
            }

            openssl_free_key($this->privateKeyId);
            return $domXml;
        }

    }

    private function assinaXmlAux(\DOMDocument $domXml)
    {
        $root = $domXml->getElementsByTagName("Header")->item(0);
        $node = $domXml->getElementsByTagName("Body")->item(0);
        $id = trim($node->getAttribute("wsu:Id"));
        $dados = $node->C14N(false, false, null, null);
        $hashValue = hash("sha1", $dados, true);
        $digValue = base64_encode($hashValue);

        $wsseSecurity = $domXml->createElement("wsse:Security");
        $root->appendChild($wsseSecurity);
        $wsseSecurity->setAttribute("xmlns:wsse", "http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd");
        $wsseSecurity->setAttribute("SOAP-ENV:mustUnderstand", "1");

        $wsseBinarySecurityToken = $domXml->createElement("wsse:BinarySecurityToken", $this->publicKey);
        $wsseSecurity->appendChild($wsseBinarySecurityToken);
        $wsseBinarySecurityToken->setAttribute("xmlns:wsu", "http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-utility-1.0.xsd");
        $wsseBinarySecurityToken->setAttribute("EncodingType", "http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-soap-message-security-1.0#Base64Binary");
        $wsseBinarySecurityToken->setAttribute("ValueType", "http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-x509-token-profile-1.0#X509v3");
        $certId = "CertId-1";
        $wsseBinarySecurityToken->setAttribute("wsu:Id", $certId);
        $wsseBinarySecurityToken->setAttribute("xmlns:wsse", "http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd");

        $signature = $domXml->createElement("ds:Signature");
        $wsseSecurity->appendChild($signature);
        $signature->setAttribute("xmlns:ds", "http://www.w3.org/2000/09/xmldsig#");
        $signature->setAttribute("Id", "Ass_".$id);

        $signedInfo = $domXml->createElement("ds:SignedInfo");
        $signature->appendChild($signedInfo);

        $novaTag = $domXml->createElement("ds:CanonicalizationMethod");
        $signedInfo->appendChild($novaTag);
        $novaTag->setAttribute("Algorithm", "http://www.w3.org/2001/10/xml-exc-c14n#");

        $novaTag = $domXml->createElement("ds:SignatureMethod");
        $signedInfo->appendChild($novaTag);
        $novaTag->setAttribute("Algorithm", "http://www.w3.org/2000/09/xmldsig#rsa-sha1");

        $reference = $domXml->createElement("ds:Reference");
        $signedInfo->appendChild($reference);
        $reference->setAttribute("URI", "#".$id);

        $transforms = $domXml->createElement("ds:Transforms");
        $reference->appendChild($transforms);

        $novaTag = $domXml->createElement("ds:Transform");
        $transforms->appendChild($novaTag);
        $novaTag->setAttribute("Algorithm", "http://www.w3.org/2001/10/xml-exc-c14n#");

        /*$novaTag = $domXml->createElement("ds:Transform");
        $transforms->appendChild($novaTag);
        $novaTag->setAttribute("Algorithm", "http://www.w3.org/TR/2001/REC-xml-c14n-20010315");*/

        $novaTag = $domXml->createElement("ds:DigestMethod");
        $reference->appendChild($novaTag);
        $novaTag->setAttribute("Algorithm", "http://www.w3.org/2000/09/xmldsig#sha1");

        $novaTag = $domXml->createElement("ds:DigestValue", $digValue);
        $reference->appendChild($novaTag);

        $dados = $signedInfo->C14N(false, false, null, null);
        $assinatura = "";
        openssl_sign($dados, $assinatura, $this->privateKeyId);
        $signatureValue = base64_encode($assinatura);

        $novaTag = $domXml->createElement("ds:SignatureValue", $signatureValue);
        $signature->appendChild($novaTag);

        $keyInfo = $domXml->createElement("ds:KeyInfo");
        $signature->appendChild($keyInfo);
        $keyInfo->setAttribute("Id", "KeyId-1");

        $wsseSecurityTokenReference = $domXml->createElement("wsse:SecurityTokenReference");
        $keyInfo->appendChild($wsseSecurityTokenReference);
        $wsseSecurityTokenReference->setAttribute("xmlns:wsu", "http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-utility-1.0.xsd");
        $wsseSecurityTokenReference->setAttribute("wsu:Id", "STRId-1");
        $wsseSecurityTokenReference->setAttribute("xmlns:wsse", "http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd");

        $wsseReference = $domXml->createElement("wsse:Reference");
        $wsseSecurityTokenReference->appendChild($wsseReference);
        $wsseReference->setAttribute("URI", "#".$certId);
        $wsseReference->setAttribute("ValueType", "http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-x509-token-profile-1.0#X509v3");
        $wsseReference->setAttribute("xmlns:wsse", "http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd");

        return $domXml;
    }

    private function getPublicKey($key)
    {
        $data = "";
        $arCert = explode("\n", $key);
        foreach ($arCert as $curData) {
            //remove a tag de inicio e fim do certificado
            if (strncmp($curData, "-----BEGIN CERTIFICATE", 22) != 0 &&
                strncmp($curData, "-----END CERTIFICATE", 20) != 0 ) {
                //carrega o resultado numa string
                $data .= trim($curData);
            }
        }
        return $data;
    }

}